<?php


class RSPW_Customer_Role_Rule implements RSPW_Rule {

	/**
	 * @param array $rule
	 * @param array $package
	 *
	 * @return boolean
	 */
	public function validate( $rule, $package ) {
		$rule_role     = $rule['value_customer_role'];
		$customer_role = WC()->customer->get_role();
		$operator      = RSPW_Operators_Factory::make( $rule['operator'] );
		return $operator->match( $customer_role, $rule_role );
	}

	/**
	 * @return array
	 */
	public function get_operators_labels() {
		return array(
			RSPW_Operators_Factory::OPERATOR_IS     => __( 'is', 'restricted-shipping-and-payment-for-woocommerce' ),
			RSPW_Operators_Factory::OPERATOR_IS_NOT => __( 'is not', 'restricted-shipping-and-payment-for-woocommerce' ),
		);
	}

	/**
	 * @return array
	 */
	public function get_meta_box_fields() {
		return array(
			array(
				'name'             => __( 'Customer Role', 'restricted-shipping-and-payment-for-woocommerce' ),
				'desc'             => __( 'Select customer role', 'restricted-shipping-and-payment-for-woocommerce' ),
				'id'               => 'value_customer_role',
				'type'             => 'select',
				'show_option_none' => false,
				'options'          => $this->roles(),
				'classes'          => 'value_field customer_role',
			),
		);
	}

	/**
	 * @return array
	 */
	private function roles() {
		if ( ! function_exists( 'get_editable_roles' ) ) {
			return array();
		}

		$editable_roles = array();

		$roles = get_editable_roles();
		foreach ( $roles as $key => $role ) {
			$editable_roles[ $key ] = $role['name'];
		}

		return $editable_roles;
	}
}
