<?php


class RSPW_Package_Weight_Rule implements RSPW_Rule {



	/**
	 * @param array $rule
	 * @param array $package
	 *
	 * @return boolean
	 */
	public function validate( $rule, $package ) {
		$rule_weight    = floatval( wc_get_weight( $rule['value_package_weight'], 'kg', 'kg' ) );
		$package_weight = floatval( $this->get_package_weight( $package ) );
		$operator       = RSPW_Operators_Factory::make( $rule['operator'] );
		return $operator->match( $package_weight, $rule_weight );
	}

	/**
	 * @return array
	 */
	public function get_operators_labels() {
		return array(
			RSPW_Operators_Factory::OPERATOR_LT    => __( 'less than', 'restricted-shipping-and-payment-for-woocommerce' ),
			RSPW_Operators_Factory::OPERATOR_EQUAL => __( 'equal to', 'restricted-shipping-and-payment-for-woocommerce' ),
			RSPW_Operators_Factory::OPERATOR_GT    => __( 'more than', 'restricted-shipping-and-payment-for-woocommerce' ),
		);
	}

	/**
	 * @param $package
	 *
	 * @return float|int
	 */
	private function get_package_weight( $package ) {
		$total_weight = 0;
		foreach ( $package as $content ) {
			/** @var WC_Product_Simple $product */
			$product       = $content['data'];
			$quantity      = $content['quantity'];
			$total_weight += floatval( $product->get_weight( 'float' ) * $quantity );
		}
		return wc_get_weight( $total_weight, 'kg' );
	}


	/**
	 * @return array
	 */
	public function get_meta_box_fields() {
		return array(
			array(
				'name'    => __( 'Package Weight', 'restricted-shipping-and-payment-for-woocommerce' ),
				'id'      => 'value_package_weight',
				'type'    => 'text_small',
				'classes' => 'value_field package_weight',
			),
		);
	}
}
