<?php

abstract class RSPW_Filter {


	/**
	 * @param $condition
	 * @param $package
	 *
	 * @return bool|string
	 */
	protected function is_condition_match( $condition, $package ) {
		$rules = $this->get_rules( $condition );

		if ( ! is_array( $rules ) ) {
			return false;
		}

		$rule_evaluation_operator = $this->get_rules_evaluation_operator( $condition );

		$rules_count         = count( $rules );
		$matched_rules_count = 0;
		foreach ( $rules as $rule ) {
			if ( ! is_array( $rule ) || ! isset( $rule['rule_type'] ) ) {
				continue;
			}

			$rule_type      = $rule['rule_type'];
			$rule_validator = RSPW_Rules_Factory::make( $rule_type );

			if ( 'AND' === $rule_evaluation_operator ) {
				if ( $rule_validator->validate( $rule, $package ) ) {
					$matched_rules_count++;
				}
			} else { // OR: only matching one rule is enough.
				if ( $rule_validator->validate( $rule, $package ) ) {
					return true;
				}
			}
		}

		if ( 'AND' === $rule_evaluation_operator ) {
			return $rules_count === $matched_rules_count;
		}

		return false;
	}

	/**
	 * @param $condition
	 *
	 * @return mixed
	 */
	protected function get_rules( $condition ) {
		/** @var WP_Post $condition */
		$rules = get_post_meta( $condition->ID, $this->get_filter_rules_key(), true );
		return $rules;
	}

	private function get_rules_evaluation_operator( $condition ) {
		/** @var WP_Post $condition */
		$rules_evaluation_operator = get_post_meta( $condition->ID, 'rules_evaluation_operator', true );
		if ( ! $rules_evaluation_operator ) {
			return 'OR';
		}
		return $rules_evaluation_operator;
	}

	/**
	 * @return array
	 */
	protected function get_conditions() {
		$conditions = get_posts(
			array(
				'post_type'      => $this->get_conditions_post_type(),
				'posts_per_page' => -1,
			)
		);
		return $conditions;
	}

	/**
	 * @return string
	 */
	abstract protected function get_filter_rules_key();

	/**
	 * @return string
	 */
	abstract protected function get_conditions_post_type();
}
