<?php
/**
 * The class to actually restrict the shipping methods
 *
 * @link       https://wpruby.com
 * @since      1.0.0
 *
 * @package    Restricted_Shipping_And_Payment_For_Woocommerce
 * @subpackage Restricted_Shipping_And_Payment_For_Woocommerce/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Restricted_Shipping_And_Payment_For_Woocommerce
 * @subpackage Restricted_Shipping_And_Payment_For_Woocommerce/public
 * @author     WPRuby <info@wpruby.com>
 */
class RSPW_Payment_Filter extends RSPW_Filter {


	/**
	 * RSPW_Payment_Filter constructor.
	 */
	public function __construct() {
		add_filter( 'woocommerce_available_payment_gateways', array( $this, 'exclude_payment_gateways' ) );
	}

	/**
	 * Filter payment gateways when restrictions apply.
	 *
	 * @param  array   $gateways
	 * @param  boolean $bypass
	 * @return array
	 */
	public function exclude_payment_gateways( $gateways, $bypass = false ) {
		if ( ! $bypass && ! is_checkout() && ! defined( 'WOOCOMMERCE_CHECKOUT' ) ) {
			return $gateways;
		}

		if ( is_checkout() ) {
			$package    = WC()->cart->get_cart_contents();
			$conditions = $this->get_conditions();
			/** @var WP_Post $condition */
			foreach ( $conditions as $condition ) {
				$condition_gateways = $this->get_gateways( $condition );
				$cart_message       = get_post_meta( $condition->ID, 'cart_message', true );
				$cart_message_type  = get_post_meta( $condition->ID, 'cart_message_type', true );
				foreach ( $gateways as $gateway_id => $gateway ) {
					if ( in_array( $gateway_id, $condition_gateways, true ) ) {
						if ( $this->is_condition_match( $condition, $package ) ) {
							if ( $cart_message && trim( $cart_message ) !== '' ) {
								if ( ! wc_has_notice( $cart_message, $cart_message_type ) ) {
									wc_add_notice( $cart_message, $cart_message_type );
								}
							}
							unset( $gateways[ $gateway_id ] );
						}
					}
				}
			}
		}

		return $gateways;
	}

	/**
	 * @param $condition
	 *
	 * @return mixed
	 */
	private function get_gateways( $condition ) {
		/** @var WP_Post $condition */
		$gateways = get_post_meta( $condition->ID, 'excluded_payment_gateways', true );
		return $gateways;
	}



	private function get_number_of_completed_orders_by_user( $package ) {
		$current_user        = intval( $package['user']['ID'] );
		$number_of_orders    = wc_get_customer_order_count( $current_user );
		$args                = array(
			'customer_id' => $current_user,
			'post_status' => 'cancelled',
			'post_type'   => 'shop_order',
			'return'      => 'ids',
		);
		$numorders_cancelled = count( wc_get_orders( $args ) );
		$num_not_cancelled   = $number_of_orders - $numorders_cancelled;
		return $num_not_cancelled;
	}

	/**
	 * @inheritDoc
	 */
	protected function get_filter_rules_key() {
		return 'payment_condition_rules';
	}

	/**
	 * @inheritDoc
	 */
	protected function get_conditions_post_type() {
		return RSPW_PAYMENT_CONDITION;
	}
}


new RSPW_Payment_Filter();
